//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/View/Realspace/RealSpaceCanvas.h
//! @brief     Defines class RealSpaceCanvas
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#ifndef BORNAGAIN_GUI_VIEW_REALSPACE_REALSPACECANVAS_H
#define BORNAGAIN_GUI_VIEW_REALSPACE_REALSPACECANVAS_H

#include <QWidget>
#include <memory>

class SampleModel;
class CautionSign;
class SessionItem;
namespace GUI::RealSpace {
class Widget3D;
class Model;
} // namespace GUI::RealSpace

//! Size and thickness information of layers
struct SceneGeometry {
    double layerSize = 100.0;
    double topOrBottomLayerThickness = 25.0;
    double layerMinimumThickness = 2.0;
};

//! Provides 3D object generation for RealSpace presentation.
class RealSpaceCanvas : public QWidget {
    Q_OBJECT

public:
    RealSpaceCanvas(SampleModel* sampleModel, QWidget* parent = nullptr);

    void setCurrentItem(SessionItem* item);
    void defaultView();
    void sideView();
    void topView();
    void changeLayerSize(double layer_size_scale);
    void savePicture();

protected:
    virtual void showEvent(QShowEvent*) override;
    virtual void hideEvent(QHideEvent*) override;

private:
    void updateScene();
    void resetScene();
    void onRowsAboutToBeRemoved(const QModelIndex& parent, int first, int last);
    void setConnected(SampleModel* model, bool makeConnected);
    void savePicture(const QPixmap& pixmap);

    SampleModel* m_sampleModel;
    GUI::RealSpace::Widget3D* m_view;
    std::unique_ptr<GUI::RealSpace::Model> m_realSpaceModel;
    SceneGeometry m_sceneGeometry;
    CautionSign* m_cautionSign;
    SessionItem* m_currentItem;
};

#endif // BORNAGAIN_GUI_VIEW_REALSPACE_REALSPACECANVAS_H
